from webex_skills.api import SimpleAPI
from webex_skills.dialogue import responses
from webex_skills.models.mindmeld import DialogueState

api = SimpleAPI()


@api.handle(default=True)
async def greet(current_state: DialogueState) -> DialogueState:
    text = 'Hello I am a super simple skill'
    new_state = current_state.copy()

    new_state.directives = [
        responses.Reply(text),
        responses.Speak(text),
        responses.Sleep(10),
    ]

    return new_state



from typing import Awaitable, Callable, Dict, Union

from .dialogue.rules import MMDialogueStateRule, SimpleDialogueStateRule
from .models.mindmeld import DialogueState, ProcessedQuery

DialogueQuery = Union[ProcessedQuery, str]
SimpleHandler = Callable[[DialogueState], Awaitable[DialogueState]]
QueryHandler = Callable[[DialogueState, DialogueQuery], Awaitable[DialogueState]]

DialogueHandler = Union[SimpleHandler, QueryHandler]
DialogueRule = Union[SimpleDialogueStateRule, MMDialogueStateRule]

RuleMap = Dict[DialogueRule, DialogueHandler]



from ..models.mindmeld import DialogueState as DialogueState, ProcessedQuery as ProcessedQuery
from ..types import (
    DialogueHandler as DialogueHandler,
    DialogueQuery as DialogueQuery,
    QueryHandler as QueryHandler,
    SimpleHandler as SimpleHandler,
)
from .rules import MMDialogueStateRule as MMDialogueStateRule, SimpleDialogueStateRule as SimpleDialogueStateRule
from typing import Any, Dict, Optional, Protocol

class MissingHandler(Exception): ...

NLPRuleMap = Dict[MMDialogueStateRule, QueryHandler]
SimpleRuleMap = Dict[SimpleDialogueStateRule, SimpleHandler]

class DialogueManager(Protocol):
    rules: Dict[Any, Any]

class MMDialogueManager:
    rules: Any
    default_handler: Any
    def __init__(self, rules: Optional[NLPRuleMap] = ..., default_handler: Optional[QueryHandler] = ...) -> None: ...
    def get_handler(self, query: ProcessedQuery, target_state: Optional[str] = ...) -> Optional[QueryHandler]: ...
    def add_rule(
        self,
        *,
        name: Optional[str] = ...,
        default: bool = ...,
        domain: Any | None = ...,
        intent: Any | None = ...,
        entities: Any | None = ...,
        targeted_only: bool = ...
    ): ...
    async def handle(self, query: DialogueQuery, current_state: DialogueState): ...

class SimpleDialogueManager:
    rules: Any
    default_handler: Any
    def __init__(
        self, rules: Optional[SimpleRuleMap] = ..., default_handler: Optional[SimpleHandler] = ...
    ) -> None: ...
    def add_rule(
        self, *, name: Optional[str] = ..., pattern: Optional[str] = ..., default: bool = ..., targeted_only: bool = ...
    ): ...
    def get_handler(self, query: str, target_state: Optional[str] = ...) -> Optional[SimpleHandler]: ...
    async def handle(self, query: str, current_state: DialogueState): ...



import typer

from .crypto import app as crypto_app
from .nlp import app as nlp_app
from .project import app as project_app
from .remote import remote as remote_app
from .skill import app as skill_app

app = typer.Typer()

app.add_typer(skill_app, name="skills")
app.add_typer(crypto_app, name="crypto")
app.add_typer(nlp_app, name='nlp')
app.add_typer(project_app)
app.add_typer(remote_app)


def main():
    app()


if __name__ == '__main__':
    main()



# pylint:disable=no-name-in-module

from typing import Any, Dict
from typing import List as _List
from typing import Optional, Union

from pydantic import BaseModel

PayloadDict = Dict[str, Any]
Payload = Union[PayloadDict, _List[Any]]


class SkillDirective(BaseModel):
    name: str
    type: str
    payload: Optional[Payload]

    def dict(  # pylint:disable=useless-super-delegation, unused-argument
        self, *args, exclude_none=True, **kwargs
    ) -> PayloadDict:
        return super().dict(*args, **kwargs, exclude_none=True)


class ViewDirective(SkillDirective):
    type: str = 'view'


class ActionDirective(SkillDirective):
    type: str = 'action'


class Listen(ActionDirective):
    name: str = 'listen'


class Reply(ActionDirective):
    name: str = 'reply'

    def __init__(self, text: str):
        super().__init__(payload={'text': text})


class Speak(ActionDirective):
    name: str = 'speak'

    def __init__(self, text):
        super().__init__(payload={'text': text})


class Sleep(ActionDirective):
    name: str = 'sleep'
    payload: Dict[str, int]

    def __init__(self, delay: int = 0):
        super().__init__(payload={'delay': delay})


class DisplayWebView(ActionDirective):
    name: str = 'display-web-view'

    def __init__(self, url: str, title: Optional[str]):
        super().__init__(payload={'url': url, 'title': title})


class ClearWebView(ActionDirective):
    name: str = 'clear-web-view'


class UIHint(ViewDirective):
    name: str = 'ui-hint'

    def __init__(self, texts, prompt, display_immediately):
        super().__init__(payload={'texts': texts, 'prompt': prompt, 'display_immediately': display_immediately})


class AsrHint(ActionDirective):
    name: str = 'asr-hint'

    def __init__(self, texts):
        super().__init__(payload={'texts': texts})


class AssistantEvent(ActionDirective):
    name: str = 'assistant-event'



import typing

from starlette.requests import ClientDisconnect
from starlette.types import ASGIApp, Message, Receive, Send


class BaseReceiver:
    def __init__(self, app: ASGIApp, receive: Receive, send: Send) -> None:
        self.app = app
        self.receive = receive
        self.send = send

    async def stream_body(self, message: Message) -> typing.AsyncGenerator[bytes, None]:
        while True:
            if message["type"] == "http.disconnect":
                raise ClientDisconnect()
            body = message.get("body", b"")
            if body:
                yield body
            if not message.get("more_body", False):
                break

            message = await self.receive()
        yield b""

    async def message_body(self, message: Message) -> bytes:
        chunks = [chunk async for chunk in self.stream_body(message)]
        return b"".join(chunks)



import base64
from typing import Optional

from fastapi import FastAPI
from pydantic import BaseSettings
from starlette.middleware import Middleware
from starlette.responses import JSONResponse

from ..crypto import verify_signature
from ..crypto.messages import decrypt, load_private_key
from ..models.http import SkillInvokeRequest, SkillInvokeResponse
from ..settings import SkillSettings
from .middlewares import DecryptionMiddleware
from .middlewares.signing import SignatureMiddleware


class BaseAPI(FastAPI):
    def __init__(self, nlp=None, dialogue_manager=None, settings: Optional[BaseSettings] = None, **kwargs):
        self.settings = settings or SkillSettings()
        self.private_key = None
        self.secret = None
        self.nlp = nlp
        self.dialogue_manager = dialogue_manager

        middleware = kwargs.pop('middlewares', [])

        if self.settings.use_encryption:
            self.private_key = load_private_key(self.settings.private_key_path.read_bytes())
            self.secret = self.settings.secret.encode('utf-8')
            # NOTE: The order here is significant. We sign the encrypted message so we want our signature
            # verification to run _before_ the decryption middleware which will alter the message to the
            # decrypted version.
            middleware = [
                Middleware(SignatureMiddleware, secret=self.secret),
                Middleware(DecryptionMiddleware, private_key=self.private_key),
                *middleware,
            ]

        super().__init__(**kwargs, middleware=middleware)
        self.router.add_api_route('/parse', self.parse, methods=['POST'], response_model=SkillInvokeResponse)
        self.router.add_api_route('/parse', self.check, methods=['GET'])

    async def parse(self, request: SkillInvokeRequest):
        pass

    # This essentially only works because there's no body and thus no call to receive()
    # we should revisit this after seeing if the middlewares can be replaced with custom request types
    async def check(self, signature: str, message: str):
        message_bytes = message.encode('utf-8')
        signature: bytes = base64.b64decode(signature.encode('utf-8'))

        if not verify_signature(secret=self.secret, message=message_bytes, signature=signature):
            return JSONResponse({'message': 'Failed to verify signature'}, status_code=400)
        try:
            challenge = decrypt(private_key=self.private_key, message=message_bytes)
        except ValueError:
            return JSONResponse({'message': 'Failed to decrypt message'}, status_code=400)
        return {'challenge': challenge}



from webex_skills.supress_warnings import suppress_warnings


def create_nlp(app_path):
    try:
        with suppress_warnings():
            from mindmeld.components.nlp import NaturalLanguageProcessor  # pylint:disable=import-outside-toplevel
    except ImportError as import_exc:
        error_text = 'You must install the extras package webex-assistant-sdk[mindmeld] to use NLP commmands'
        raise ImportError(error_text) from import_exc
    nlp = NaturalLanguageProcessor(app_path=app_path)
    return nlp



from pathlib import Path
import secrets
from typing import Optional, Union

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import BestAvailableEncryption, NoEncryption

EncryptionTypes = Union[BestAvailableEncryption, NoEncryption]


def generate_keys(priv_path: Path, pub_path: Path, encryption: Optional[EncryptionTypes] = None):

    if not encryption:
        encryption = NoEncryption()
    private_key = rsa.generate_private_key(65537, 4096)
    public_key: rsa.RSAPublicKey = private_key.public_key()

    priv_path.write_bytes(
        private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=encryption,
        )
    )

    pub_path.write_bytes(
        public_key.public_bytes(
            encoding=serialization.Encoding.PEM, format=serialization.PublicFormat.SubjectPublicKeyInfo
        )
    )


def generate_secret():
    return secrets.token_urlsafe(16)



import warnings


class suppress_warnings:  # pylint:disable=invalid-name
    def __init__(self):
        self._showwarning = None

    def showwarning(self, *args, **kwargs):
        pass

    def __enter__(self):
        self._showwarning = warnings.showwarning
        warnings.showwarning = self.showwarning

    def __exit__(self, exc_type, exc_val, exc_tb):
        warnings.showwarning = self._showwarning



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['webex_skills',
 'webex_skills.api',
 'webex_skills.api.middlewares',
 'webex_skills.cli',
 'webex_skills.crypto',
 'webex_skills.dialogue',
 'webex_skills.models',
 'webex_skills.static']

package_data = \
{'': ['*'],
 'webex_skills.static': ['default_domains/greeting/exit/*',
                         'default_domains/greeting/greet/*']}

install_requires = \
['cryptography>=35.0.0,<36.0.0',
 'fastapi>=0.68.1,<0.69.0',
 'pydantic[dotenv]>=1.8.2,<2.0.0',
 'requests>=2.22.0,<3.0.0',
 'typer>=0.4.0,<0.5.0',
 'uvicorn>=0.15.0,<0.16.0']

extras_require = \
{'mindmeld': ['mindmeld>=4.4.0,<5.0.0']}

entry_points = \
{'console_scripts': ['webex-skills = webex_skills.cli:main']}

setup_kwargs = {
    'name': 'webex-skills',
    'version': '2.0.10',
    'description': 'An SDK for skills for the Webex Assistant.',
    'long_description': '# Webex Assistant Skills SDK\n\nThe Webex Skills SDK is designed to simplify the process of creating a Webex Assistant Skill. It provides some\ntools that help to easily set up a template skill, deal with encryption and test the skill locally, among other\nthings.\n\nThis is a simplified version of the documentation, for more information visit the official  \n[Webex Assistant Skills SDK website](https://developer.webex.com/docs/api/guides/webex-assistant-skills-guide).\n\n## Installing the SDK\n\nThis SDK supports `Python 3.7` and above. If you want to be able to build\n[MindMeld Skills](https://developer.webex.com/docs/api/guides/webex-assistant-skills-guide#building-a-mindmeld-skill),\nyou will need to use `Python 3.7` for compatibility with the [MindMeld library](https://www.mindmeld.com/).\n\n### Using pip\n\nCreate a virtual environment with `Python 3.7`:\n\n```python\npyenv install 3.7.5\npyenv virtualenv 3.7.5 webex-skills\npyenv local webex-skills\n```\n\nInstall using `pip`:\n\n```python\npip install webex-skills\n```\n\nIn order to build `MindMeld Skills` you need the `mindmeld` extra:\n\n```python\npip install \'webex-skills[mindmeld]\'\n```\n\n### Install from Source\n\nYou can install from source using `Poetry`. Set up python 3.7.5 locally:\n\n```python\npyenv install 3.7.5\npyenv local 3.7.5\n```\n\nNow you can run `Poetry`\'s `install` command:\n\n```python\npoetry install\n```\n\nIn order to build `MindMeld Skills` you need the `mindmeld` extra:\n\n```python\npoetry install --extras "mindmeld"\n```\n\n\n## Building Skills\n\nYou can follow the next guides for building your first skills:\n\n- [Simple Skills vs MindMeld Skills](https://developer.webex.com/docs/api/guides/webex-assistant-skills-guide#simple-skills-vs-mindmeld-skills)\n- [Building a Simple Skill](https://developer.webex.com/docs/api/guides/webex-assistant-skills-guide#building-a-simple-skill)\n- [Building a MindMeld Skill](https://developer.webex.com/docs/api/guides/webex-assistant-skills-guide#building-a-mindmeld-skill)\n',
    'author': 'James Snow',
    'author_email': 'jasnow@cisco.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/cisco/webex-assistant-sdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import json
from pathlib import Path

import typer


def get_skill_config(name=None):
    app_dir = Path(typer.get_app_dir('skills-cli', force_posix=True))
    config_file = app_dir / 'config.json'

    if not app_dir.exists():
        typer.echo('Creating default configuration')
        app_dir.mkdir(parents=True)
        config = {}
    else:
        config = json.loads(config_file.read_text(encoding='utf-8')) or {}

    remotes = config.get('remotes', {})
    if not name:
        return remotes

    remote_config = remotes.get(name)
    if not remote_config:
        typer.secho(f'No configured remote with the name {name} found', color=typer.colors.RED, err=True)
        raise typer.Exit(1)

    return remote_config
